# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaMess -- Fabric Automation Helpers
#  Copyright © 2024 Lance Edgar
#
#  This file is part of Wutta Framework.
#
#  Wutta Framework is free software: you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option) any
#  later version.
#
#  Wutta Framework is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#  more details.
#
#  You should have received a copy of the GNU General Public License along with
#  Wutta Framework.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Misc. Utilities
"""

from pathlib import Path
from typing_extensions import Any, Mapping

from mako.template import Template


def exists(c, path):
    """
    Returns ``True`` if given path exists on the host, otherwise ``False``.
    """
    return not c.run(f'test -e {path}', warn=True).failed


def mako_renderer(c, env={}):
    """
    This returns a *function* suitable for use as a ``fabsync`` file
    renderer.  The function assumes the file is a Mako template.

    :param c: Fabric connection.

    :param env: Environment dictionary to be used as Mako template
       context.

    Typical usage is something like::

       from fabric import task
       from wuttamess import sync, util

       root = sync.make_root('files')
       env = {}

       @task
       def foo(c):

           # define possible renderers for fabsync
           renderers = {'mako': util.mako_renderer(c, env)}

           sync.check_isync(c, root, 'etc/postfix', renderers=renderers)
    """
    def render(path: Path, vars: Mapping[str, Any], **kwargs) -> bytes:
        return Template(filename=str(path)).render(**env)

    return render
